@@ -1,4 +1,5 @@
 import subprocess
+from signal import Signals
 
 
 def shorten_long_output(output, max_output_str_len=8000):
@@ -11,8 +12,18 @@ def shorten_long_output(output, max_output_str_len=8000):
 
 class FireXCalledProcessError(subprocess.CalledProcessError):
     def __str__(self):
-        mssg = ("Command '%s' returned non-zero exit status %d. " %
-                (self.cmd, self.returncode))
+        if self.returncode < 0:
+            # POSIX says this is a signal
+            try:
+                signame = Signals(-self.returncode).name
+            except ValueError:
+                signame = -self.returncode
+
+            status = f'signal {signame}'
+        else:
+            status = f'exit code {self.returncode}'
+
+        mssg = f'Command {self.cmd} exited with {status}.'
         if self.output:
             mssg += shorten_long_output(self.output)
         return mssg
