@@ -11,7 +11,7 @@ from typing import Callable, Iterable, Optional, Union
 from urllib.parse import urljoin
 from copy import deepcopy
 
-from celery.canvas import Signature
+from celery.canvas import Signature, _chain
 from celery.result import AsyncResult
 from celery.states import REVOKED
 from contextlib import contextmanager
@@ -39,6 +39,24 @@ FIREX_REVOKE_COMPLETE_EVENT_TYPE = 'task-firex-revoked'
 
 logger = get_task_logger(__name__)
 
+################################################################
+# Monkey patching
+_orig_chain_apply_async__ = _chain.apply_async
+
+
+def _chain_apply_async(self: _chain, *args: tuple, **kwargs: dict) -> AsyncResult:
+    try:
+        chain_depth = 0
+        for task in self.tasks:
+            task.kwargs['chain_depth'] = chain_depth
+            chain_depth += 1
+    except AttributeError:
+        pass
+    return _orig_chain_apply_async__(self, *args, **kwargs)
+
+
+_chain.apply_async = _chain_apply_async
+
 
 class TaskContext:
     pass
@@ -530,6 +548,36 @@ class FireXTask(Task):
         if raise_exception:
             raise e
 
+    def _process_result(self, result, extra_events: Optional[dict] = None):
+        # Need to update the dict with the results, if @results was used
+        if isinstance(result, dict):
+            self.context.bog.update(result)
+
+        # run any post converters attached to this task
+        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=False, **self.bag)
+        self.context.bog.update(converted)
+
+        if isinstance(result, dict):
+            # update the results with changes from converters
+            result = {k: v for k, v in self.bag.items() if k in result}
+
+        # give sub-classes a chance to do something with the results
+        self.post_task_run(result, extra_events=extra_events)
+
+        return self.bag
+
+    def convert_results_if_returns_defined_by_task_definition(self, result):
+        # If @returns decorator was used, we don't need to convert since that's taken care by the decorator
+        if not self._decorated_return_keys and self._task_return_keys:
+            # This is only used if the @app.task(returns=) is used
+            result = self.convert_returns_to_dict(self._task_return_keys, result)
+        return result
+
+    def final_call(self, *args, **kwargs):
+        result = super(FireXTask, self).__call__(*self.args, **self.kwargs)
+        converted_result = self.convert_results_if_returns_defined_by_task_definition(result)
+        return self._process_result(converted_result)
+
     def _process_arguments_and_run(self, *args, **kwargs):
         # Organise the input args by creating a BagOfGoodies
         self.context.bog = BagOfGoodies(self.sig,
@@ -544,27 +592,7 @@ class FireXTask(Task):
         # give sub-classes a chance to do something with the args
         self.pre_task_run()
 
-        result = super(FireXTask, self).__call__(*self.args, **self.kwargs)
-
-        if not self._decorated_return_keys and self._task_return_keys:
-            result = self.convert_returns_to_dict(self._task_return_keys, result)
-
-        # Need to update the dict with the results, if @results was used
-        if isinstance(result, dict):
-            self.context.bog.update(result)
-
-        # run any post converters attached to this task
-        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=False, **self.bag)
-        self.context.bog.update(converted)
-
-        if isinstance(result, dict):
-            # update the results with changes from converters
-            result = {k: v for k, v in self.bag.items() if k in result}
-
-        # give sub-classes a change to do something with the results
-        self.post_task_run(result)
-
-        return self.bag
+        return self.final_call(*self.args, **self.kwargs)
 
     def retry(self, *args, **kwargs):
         # Adds some logging to the original task retry
@@ -901,7 +929,7 @@ class FireXTask(Task):
 
         result = AsyncResult(task_uid)
         if block:
-            logger.print(f'Waiting for results of non-child task {get_result_logging_name(result)}')
+            logger.debug(f'Waiting for results of non-child task {get_result_logging_name(result)}')
             wait_on_async_results_and_maybe_raise(results=result, caller_task=self, **kwargs)
         return result
 
